package core.notevault.sync.synchronisation.songs;

import android.content.Context;
import android.os.AsyncTask;
import android.widget.Toast;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.data.sync.SyncStatus;
import core.notevault.sync.synchronisation.SyncService;
import core.notevault.sync.synchronisation.songs.creation.SongCreationBatchRequest;
import core.notevault.sync.synchronisation.songs.creation.SongCreationBatchResponse;
import core.notevault.sync.synchronisation.songs.creation.SongCreationRequest;
import core.notevault.sync.synchronisation.songs.deletion.SongDeletionBatchRequest;
import core.notevault.sync.synchronisation.songs.deletion.SongDeletionBatchResponse;
import core.notevault.sync.synchronisation.songs.modification.SongModificationBatchRequest;
import core.notevault.sync.synchronisation.songs.modification.SongModificationBatchResponse;
import core.notevault.sync.synchronisation.songs.modification.SongModificationRequest;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

public class SongSyncWorker {
    private SyncService syncService;
    private MusicNoteDAO database;
    private Context context;
    private ExecutorService executorService = Executors.newSingleThreadExecutor();

    public SongSyncWorker(SyncService syncService, MusicNoteDAO database, Context context) {
        this.syncService = syncService;
        this.database = database;
        this.context = context;
    }

    private SongCreationBatchRequest buildCreationRequest() {
        List<MusicNote> created_songs = database.getSongsWithSyncStatus(SyncStatus.CREATED);
        List<SongCreationRequest> songCreationRequests = new ArrayList<>();
        for(MusicNote musicNote : created_songs) {
            songCreationRequests.add(new SongCreationRequest(musicNote.getMusicNoteId(), musicNote.getTitle(), musicNote.getComposer(), musicNote.getGenre(), musicNote.getYear()));
        }
        return new SongCreationBatchRequest(songCreationRequests);
    }

    public void syncSongCreations() {
        SongCreationBatchRequest request = buildCreationRequest();
        syncService.performSongCreation(request).enqueue(new Callback<SongCreationBatchResponse>() {
            @Override
            public void onResponse(Call<SongCreationBatchResponse> call, Response<SongCreationBatchResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    executorService.execute(() -> {
                        database.storeSongSyncResponses(response.body().getSongs());
                    });
                } else {
                    Toast.makeText(context, "Song creation failed", Toast.LENGTH_LONG).show();
                }
            }

            @Override
            public void onFailure(Call<SongCreationBatchResponse> call, Throwable throwable) {
                Toast.makeText(context, "Song creation failed", Toast.LENGTH_LONG).show();
            }
        });
    }

    private SongModificationBatchRequest buildModificationRequest() {
        List<MusicNote> modified_songs = database.getSongsWithSyncStatus(SyncStatus.MODIFIED);
        List<SongModificationRequest> songModificationRequests = new ArrayList<>();
        for(MusicNote musicNote : modified_songs) {
            songModificationRequests.add(new SongModificationRequest(musicNote.getServerID(), musicNote.getTitle(), musicNote.getComposer(), musicNote.getGenre(), musicNote.getYear()));
        }
        return new SongModificationBatchRequest(songModificationRequests);
    }

    public void syncSongModifications() {
        SongModificationBatchRequest request = buildModificationRequest();
        syncService.performSongModification(request).enqueue(new Callback<SongModificationBatchResponse>() {
            @Override
            public void onResponse(Call<SongModificationBatchResponse> call, Response<SongModificationBatchResponse> response) {
                executorService.execute(() -> {
                    database.storeSongSyncModifyResponses(response.body().getUpdated_songs(), SyncStatus.SYNCED, LocalDateTime.now());
                });
            }

            @Override
            public void onFailure(Call<SongModificationBatchResponse> call, Throwable throwable) {
                Toast.makeText(context, "Sync of Modified Songs failed", Toast.LENGTH_LONG).show();
            }
        });
    }

    private SongDeletionBatchRequest buildDeletionBatchRequest() {
        List<MusicNote> deleted_songs = database.getSongsWithSyncStatus(SyncStatus.DELETED);
        List<String> deleted_song_ids = deleted_songs.stream().map(MusicNote::getServerID).collect(Collectors.toList());
        return new SongDeletionBatchRequest(deleted_song_ids);
    }

    public void syncSongDeletions() {
        SongDeletionBatchRequest request = buildDeletionBatchRequest();
        syncService.performSongDeletion(request).enqueue(new Callback<SongDeletionBatchResponse>() {
            @Override
            public void onResponse(Call<SongDeletionBatchResponse> call, Response<SongDeletionBatchResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    executorService.execute(() -> {
                        database.deleteSongs(response.body().getDeletedSongs());
                    });
                }
            }

            @Override
            public void onFailure(Call<SongDeletionBatchResponse> call, Throwable throwable) {
                Toast.makeText(context, "Sync of Deleted Songs failed", Toast.LENGTH_LONG).show();
            }
        });
    }
}
